"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPrepended = void 0;
// @ts-ignore
const defaults_1 = __importDefault(require("metro-config/src/defaults/defaults"));
// @ts-ignore
const getPreludeCode_1 = __importDefault(require("metro/src/lib/getPreludeCode"));
// @ts-ignore
const countLines_1 = __importDefault(require("metro/src/lib/countLines"));
const getDependencies_1 = require("./getDependencies");
const getPrepended = async (options) => {
    const { config, transformOptions, dependencyGraph, platform, transformer, extraVars, } = options;
    const polyfillModuleNames = config.serializer
        .getPolyfills({ platform })
        .concat(config.serializer.polyfillModuleNames);
    const dependencies = await (0, getDependencies_1.getDependencies)({
        entryFiles: [defaults_1.default.moduleSystem, ...polyfillModuleNames],
        config,
        dependencyGraph,
        transformer,
        transformOptions: {
            ...transformOptions,
            type: 'script',
        }
    });
    const getPrelude = () => {
        const code = (0, getPreludeCode_1.default)({
            isDev: transformOptions.dev,
            globalPrefix: config.transformer.globalPrefix,
            extraVars,
        });
        const name = '__prelude__';
        return {
            dependencies: new Map(),
            getSource: () => Buffer.from(code),
            inverseDependencies: new Set(),
            path: name,
            output: [
                {
                    type: 'js/script/virtual',
                    data: {
                        code,
                        lineCount: (0, countLines_1.default)(code),
                        map: [],
                    },
                },
                {
                    type: 'bytecode/script/virtual',
                    data: {
                        // TODO: hermesCompiler 有副作用, 引入后会影响当前进程.
                        // 1. 进程抛出的错误会被它捕获, 然后仅抛出有限的错误信息
                        // 2. 会产生一些奇怪的行为, 比如 babel-loader 中的 AcornParser.extend 会抛出错误:
                        // 2.1 RuntimeError: abort(TypeError: AcornParser.extend is not a function). Build with -s ASSERTIONS=1 for more info
                        // 2.2 RuntimeError: abort(TypeError: Class constructor TerminalReporter cannot be invoked without 'new'). Build with -s ASSERTIONS=1 for more info.
                        // bytecode: hermesCompiler(code, { sourceURL: '__prelude__.virtual.js' }).bytecode,
                        bytecode: Buffer.from('')
                    },
                },
            ],
        };
    };
    return [
        getPrelude(),
        ...dependencies.values(),
    ];
};
exports.getPrepended = getPrepended;
