"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExtMetroConfig = exports.skipModulePlaceHolder = void 0;
const path_1 = __importDefault(require("path"));
// @ts-ignore untyped metro-cache
const metro_cache_1 = require("metro-cache");
// @ts-ignore
const metro_config_1 = require("metro-config");
// @ts-ignore
const defaults_1 = require("metro-config/src/defaults");
const cli_plugin_metro_1 = require("@react-native-community/cli-plugin-metro");
const createModuleIdHOF_1 = require("./createModuleIdHOF");
const resolveReactNativePath_1 = __importDefault(require("./resolveReactNativePath"));
const filterModuleHOF_1 = __importDefault(require("./filterModuleHOF"));
const ramda_1 = require("ramda");
exports.skipModulePlaceHolder = {
    dependencies: [],
    output: [],
    getSource: () => Buffer.from(''),
};
// TODO: 搜索 ext-sdk 所在文件夹
function getExtMetroConfig(options) {
    const { projectRoot, mode, projectConfig, extMetroConfig: { babelPlugins, hyextTransformerConfig, filterModules = new Set(), filterModulesVersion = new Set(), SDKAlias = {}, extraPolyfills = [], extraModulesRunFirst = [], }, } = options;
    // 通过这个向transform.js 文件传值
    process.env.MATRIX_BABEL_PLUGINS = JSON.stringify(babelPlugins || []);
    process.env.MATRIX_HYEXT_TRANSFORMER_CONFIG = JSON.stringify(hyextTransformerConfig || {});
    // metro 默认值: metro-config/src/defaults/index.js
    const officialDefaultConfig = (0, defaults_1.getDefaultValues)(projectRoot);
    const officialCliConfig = (0, cli_plugin_metro_1.getDefaultConfig)({
        platforms: {},
        reactNativePath: (0, resolveReactNativePath_1.default)([process.env.MATRIX_SHARE_PROJECT_RN_ROOT, projectRoot].filter(Boolean)),
    });
    const officialConfig = (0, metro_config_1.mergeConfig)(officialDefaultConfig, officialCliConfig);
    // 所以的依赖必须要归到 watchFolder 中去, 否则找不到相关的文件, 或者文件缺失md5
    const extraWatchFolder = [
        'metro-runtime/src/modules/asyncRequire',
        '@react-native-community/cli-plugin-metro', // or come from here
    ]
        .map(xs => require.resolve(xs))
        .map(xs => require.resolve('metro-runtime/src/modules/asyncRequire', { paths: [xs] })) // watch asyncRequire 所在的文件夹
        .map(xs => path_1.default.dirname(xs));
    // watch matrix-config 所在的文件夹
    extraWatchFolder.push(path_1.default.resolve(__dirname));
    // watch metro-runtime/src 文件夹
    extraWatchFolder.push(path_1.default.join(path_1.default.dirname(require.resolve('metro-runtime/package.json')), 'src'));
    extraWatchFolder.push(...extraModulesRunFirst.map(xs => path_1.default.dirname(xs)), ...extraPolyfills.map(xs => path_1.default.dirname(xs)));
    /*
     * 开发环境下, 这个值设置为true, 否则会遇到问题:
     * https://github.com/facebook/react-native/issues/28801
     */
    let inlineRequires = true;
    let watchFolders = (0, ramda_1.uniq)([projectRoot, ...extraWatchFolder]);
    let nodeModulesPaths = [path_1.default.join(projectRoot, 'node_modules')];
    let assetRegistryPath = officialConfig.transformer.assetRegistryPath;
    let useWatchman = true;
    let createModuleIdFactory = createModuleIdHOF_1.defaultFactory;
    if (mode !== 'dev') {
        inlineRequires = false;
        assetRegistryPath = path_1.default.resolve(path_1.default.join(__dirname, 'assetRegistry.js'));
        useWatchman = false;
        createModuleIdFactory = (0, createModuleIdHOF_1.createModuleIdHOF)({ projectRoot, SDKAlias, filterModules });
        if (process.env.MATRIX_SHARE_PROJECT_RN_ROOT) {
            const shareModules = path_1.default.join(process.env.MATRIX_SHARE_PROJECT_RN_ROOT, 'node_modules');
            // 大部分包(react/react-dom/react-native/sdk)其实都在 projectRoot/node_modules 这一层,
            // 所以先搜索这个位置比较高效
            nodeModulesPaths = [shareModules, ...nodeModulesPaths];
            watchFolders.push(shareModules);
        }
    }
    const cacheVersion = (projectConfig === null || projectConfig === void 0 ? void 0 : projectConfig.cacheVersion) || 0;
    const CACHE_PATH = path_1.default.join(process.env.MATRIX_CACHE_ROOT || path_1.default.join(projectRoot, 'node_modules', '.huya-ext-cache'), 'metro');
    const hyextConfig = {
        resolver: {
            sourceExts: [...officialConfig.resolver.sourceExts, 'jsx', 'hycss'],
            platforms: ['ios', 'android', 'native', 'web'],
            nodeModulesPaths,
            blockList: [
                ...(Array.isArray(officialConfig.resolver.blockList) ? officialConfig.resolver.blockList : [officialConfig.resolver.blockList]),
                ...options.extMetroConfig.blockRegexp || [],
            ],
            useWatchman,
        },
        serializer: {
            processModuleFilter: (0, filterModuleHOF_1.default)({ projectRoot, SDKAlias, filterModules, filterModulesVersion, mode }),
            createModuleIdFactory,
            getModulesRunBeforeMainModule: (entryRelative) => {
                const officialRunBefore = officialConfig.serializer.getModulesRunBeforeMainModule(entryRelative);
                return [...officialRunBefore, ...extraModulesRunFirst];
            },
            getPolyfills: (opts) => {
                const officialPolyfills = officialConfig.serializer.getPolyfills(opts);
                return [...officialPolyfills, ...extraPolyfills];
            }
        },
        server: {
            port: options.port || 8081,
            // 中间件的执行顺序: communityCLI => metroMiddleware.
            // 这个中间件中的逻辑, 会晚于 communityCLI 的中间件执行
            enhanceMiddleware: undefined,
        },
        transformer: {
            // babelPlugins: [],
            enableBabelRCLookup: false,
            getTransformOptions: async () => ({
                transform: {
                    experimentalImportSupport: false,
                    inlineRequires,
                    unstable_disableES6Transforms: false,
                },
                preloadedModules: false,
                ramGroups: [],
            }),
            // asyncRequireModulePath: require.resolve('metro-runtime/src/modules/asyncRequire'),
            babelTransformerPath: path_1.default.join(__dirname, '../matrix-config/babelTransformer.js'),
            assetRegistryPath,
            // publicPath用来决定图片的地址, 开发环境需设置为空
            publicPath: '/',
        },
        cacheStores: cacheVersion < 0 ? [] : [new metro_cache_1.FileStore({ root: CACHE_PATH })],
        hasteMapCacheDirectory: CACHE_PATH,
        // 确保不同的小程序能产生不同的缓存值
        cacheVersion: cacheVersion.toString(),
        projectRoot,
        watchFolders,
        onReady: () => { },
        enableRootBabelRC: false,
    };
    return (0, metro_config_1.mergeConfig)(officialDefaultConfig, officialCliConfig, hyextConfig, options.extMetroConfig);
}
exports.getExtMetroConfig = getExtMetroConfig;
